/******************************************************************
Copyright (c) 1996 Novell, Inc. All Rights Reserved.

THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
TREATIES. USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
THAT CONTAINS THIS WORK. PURSUANT TO THE SDK LICENSE AGREEMENT,
NOVELL HEREBY GRANTS TO DEVELOPER A ROYALTY-FREE, NON-EXCLUSIVE
LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS PRODUCT. NOVELL
GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF
DEVELOPER'S PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO
DEVELOPER OR DEVELOPER'S CUSTOMERS WITH RESPECT TO THIS CODE.
******************************************************************/

package com.novell.nds.dirxml.driver.jdbc.db;


import java.sql.*;


/**
 * A database-independent library of useful JDBC methods.
 *  <p>
 *  To be installed on DB2 database server.  Must be compiled
 *  for 1.1.8 VM.
 */
abstract public
class DBLib
{

    //prevents javadoc of default constructor
    private
    DBLib() {}


    /**
     * Is the passed <code>String</code> is something other than <code>null</code>,
     * empty string, or whitespace?
     *
     * @param value the value in question
     * @return <code>true</code> if <code>value</code> is a value;
     * <code>false</code> otherwise
     */
    public static
    boolean
    IsValue(String value)
    {
        return !IsNotValue(value);
    }//IsValue(String)


    /**
     * Is the passed <code>String</code> is <code>null</code>,
     * empty string, or whitespace?
     *
     * @param value the value in question
     * @return <code>true</code> if <code>value</code> is not a value;
     * <code>false</code> otherwise
     */
    public static
    boolean
    IsNotValue(String value)
    {
        return (value == null) ||
               IsWhitespace(value);
    }//IsNotValue(String)


    static private
    boolean
    IsWhitespace(String s)
    {
        if(s == null)
        {
            return false;
        }

        int length;

        length = s.length();
        if(length == 0)
        {
            //treat empty string is equivalent to whitespace
            return true;
        }
        else if(length == 1)
        {
            return Character.isWhitespace(s.charAt(0));
        }
        else
        {
            char[] chars;

            chars = s.toCharArray();
            for(int c=0; c<chars.length; c++)
            {
                if(!Character.isWhitespace(chars[c]))
                {
                    return false;
                }
            }//for

            return true;
        }//else
    }//IsWhitespace()


    /**
     * Closes open connection.  Useful in a finally block.
     * Thrown <code>SQLException</code>s are ignored.
     *
     * @param connection may be <code>null</code>
     */
    static public
    Connection
    Close(Connection connection)
    {
        if(connection != null)
        {
            try
            {
                connection.close();
            }
            catch(SQLException sql)
            {
                //ignore
            }
        }//if

        return null;
    }//Close(Connection)


    /**
     * Closes open statement.  Useful in a finally block.
     * Thrown <code>SQLException</code>s are ignored.
     *
     * @param stmt may be <code>null</code>
     */
    static public
    Statement
    Close(Statement stmt)
    {
        if(stmt != null)
        {
            try
            {
                stmt.close();
            }
            catch (SQLException sql)
            {
                //ignore
            }
        }//if

        return null;
    }//Close(Statement)


    /**
     * Closes open results set.  Useful in a finally block.
     * Thrown <code>SQLException</code>s are ignored.
     *
     * @param rs may be <code>null</code>
     */
    static public
    ResultSet
    Close(ResultSet rs)
    {
        if(rs != null)
        {
            try
            {
                rs.close();
            }
            catch (SQLException sql)
            {
                //ignore
            }
        }//if

        return null;
    }//Close(ResultSet)


//----------------------------------------------------------------------
// Copied from JDBCLib and JDBCConstants

    private static final int LDAP_DN_NO_OF_DELIMITERS        = 2;
    private static final int LDAP_DN_NO_OF_ESCAPED_CHARS     = 2; //assumes won't happen very often
    private static final char LDAP_DN_DELIMITER              = '"';
    private static final char LDAP_DN_CHAR_ESCAPE            = '\\';


    /**
     * Does a character need to be escaped if included in an LDAP-delimited
     * string?
     *
     * @param ch the character to evaluate
     * @return <code>true</code> if it does;
     * <code>false</code> otherwise
     */
    static public
    boolean
    LDAPMustEscapeDelimited(char ch)
    {
        //list derived from RFC 2253
        return ((ch == LDAP_DN_DELIMITER) ||
                (ch == LDAP_DN_CHAR_ESCAPE));
    }//LDAPMustEscapeDelimited(char)
//----------------------------------------------------------------------


    /**
     * Encloses <code>undelimited</code> in double quotes "".
     * Provides a simple alternative to escaping LDAP DN characters
     * <code>{'"' | '\' | ',' | '+' | '=' | '<' | '>' | ';'}</code>.
     *
     * @param undelimited  an unquoted string; may be <code>null</code>
     * @return <code>null</code> if <code>undelimited</code> is <code>null</code>;
     *         <code>undelimited</code> enclosed in "" otherwise.
     */
    public static
    String //delimited
    DELIMIT(String undelimited)
    {
        if(undelimited == null)
        {
            return null;
        }
        else
        {
            StringBuffer delimited;
            int length;
            char ch;

            length = undelimited.length();

            delimited = new StringBuffer(length + LDAP_DN_NO_OF_DELIMITERS +
                                                  LDAP_DN_NO_OF_ESCAPED_CHARS);

            delimited.append(LDAP_DN_DELIMITER);

            for(int c=0; c<length; c++)
            {
                ch = undelimited.charAt(c);
                if(LDAPMustEscapeDelimited(ch))
                {
                    delimited.append(LDAP_DN_CHAR_ESCAPE);
                }
                delimited.append(ch);
            }//for

            delimited.append(LDAP_DN_DELIMITER);

            return delimited.toString();
        }//else
    }//DELIMIT(String)


    /**
     * Instantiates a timestamp object.
     *
     * @param odbcTimestamp a string of the format
     *   <code>yyyy-mm-dd hh:mm:ss&#46;fffffffff</code>;
     *   MUST not be <code>null</code>
     * @return will not return <code>null</code>
     * @throws IllegalArgumentException if <code>odbcTimestamp</code>
     *  does not have the format <code>yyyy-mm-dd hh:mm:ss.fffffffff</code>
     */
    static public
    java.sql.Timestamp
    NewTimestamp(String odbcTimestamp)
        throws IllegalArgumentException
    {
        return Timestamp.valueOf(odbcTimestamp);
    }


    /**
     * Instantiates a time object.
     *
     * @param odbcTime a string of the format
     *   <code>hh:mm:ss</code>;
     *   MUST not be <code>null</code>
     * @return will not return <code>null</code>
     * @throws IllegalArgumentException if <code>odbcTime</code>
     *  does not have the format <code>hh:mm:ss</code>
     */
    static public
    java.sql.Time
    NewTime(String odbcTime)
        throws IllegalArgumentException
    {
        return Time.valueOf(odbcTime);
    }


    /**
     * Instantiates a date object.
     *
     * @param odbcDate a string of the format
     *   <code>yyyy-mm-dd</code>;
     *   MUST not be <code>null</code>
     * @return will not return <code>null</code>
     * @throws IllegalArgumentException if <code>odbcDate</code>
     *  does not have the format <code>yyyy-mm-dd</code>
     */
    static public
    java.sql.Date
    NewDate(String odbcDate)
        throws IllegalArgumentException
    {
        return java.sql.Date.valueOf(odbcDate);
    }

}//class DBLib
