/******************************************************************
Copyright (c) 1996 Novell, Inc. All Rights Reserved.

THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
TREATIES. USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
THAT CONTAINS THIS WORK. PURSUANT TO THE SDK LICENSE AGREEMENT,
NOVELL HEREBY GRANTS TO DEVELOPER A ROYALTY-FREE, NON-EXCLUSIVE
LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS PRODUCT. NOVELL
GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF
DEVELOPER'S PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO
DEVELOPER OR DEVELOPER'S CUSTOMERS WITH RESPECT TO THIS CODE.
******************************************************************/

package com.novell.nds.dirxml.driver.jdbc.db;


import java.sql.Time;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.regex.Pattern;


/**
 *  An abstract database timestamp.
 */
public
class DBTimestamp
    extends AbstractTimeValue
    implements DBTimeTranslator
{


    static protected final String DEFAULT_DB_FORMAT = "odbc";
    static protected final int LEN_ODBC_TIMESTAMP  = 19;


    static private
    String
    GetNanosAsString(int inanos)
    {
        //999999999 <= x >= 0; at most 9 digits

        StringBuffer buffer;
        String nanos;

        nanos = String.valueOf(inanos);
        buffer = new StringBuffer(MAX_SCALE_NANOS);
        //prepend 0s
        for(int i=(MAX_SCALE_NANOS-nanos.length()); i>0; i--)
        {
            buffer.append(0);
        }

        buffer.append(nanos);

        //postpend 0s
        for(int i=(MAX_SCALE_NANOS - buffer.length()); i>0; i--)
        {
            buffer.append(0);
        }

        return buffer.toString();
    }//GetNanosAsString()


    static private
    boolean
    PrecisionLoss(long millis)
    {
        return ((millis%1000) != 0);
    }//PrecisionLoss(long)


    static private
    boolean
    PrecisionLoss(Timestamp ts)
    {
        if(ts == null)
        {
            return false;
        }
        else
        {
            return ((ts.getNanos() != 0) ||
                    (PrecisionLoss(ts.getTime())));
        }
    }//PrecisionLoss()


    protected boolean _precisionLost = false;
    private boolean _customImpl = true; //assume custom implementation


    //no default constructor is permitted
    private
    DBTimestamp(){};


    public
    DBTimestamp
    (
        String dbTimestamp,
        String sqlClass,
        String inFormat,
        String outFormat
    )
    {
        impl(dbTimestamp,
             sqlClass,
             inFormat,
             outFormat);
    }


    public
    void
    setDSTime(String nativeTime)
        throws ParseException
    {
        _customImpl = false;
    }//setDSTime(String)


    protected
    void
    setTimestamp(Timestamp ts)
    {
        SimpleDateFormat sdf;

        sdf = new SimpleDateFormat(FORMAT_TIMESTAMP);
        if(this._outFormat == DSTimeFormat.INT)
        {
            _time = String.valueOf(Lib.ToSeconds(ts.getTime(), null));
            if(PrecisionLoss(ts))
            {
                this._precisionLost = true;
            }
        }//if
        else if(this._outFormat == DSTimeFormat.STRING)
        {
            _time = sdf.format(ts);
            _time += GetNanosAsString(ts.getNanos());
        }
        else
        {
            throw new RuntimeException
                (Text.BROKEN_IF_ELSE);
        }
    }//setTimestamp()


    private
    void
    impl
    (
        String dbTimestamp,
        String sqlClass,
        String inFormat,
        String outFormat
    )
    {
        _time = dbTimestamp;
        _precisionLost = false;

        if(DBLib.IsNotValue(dbTimestamp))
        {
            _parseError = true;
            _errorMessage = "Null or empty string is not a valid database timestamp/date/time value.";
            addException(new IllegalArgumentException(_errorMessage));
        }
        else
        {
            dbTimestamp = dbTimestamp.trim();
            if(Pattern.matches("-?\\d+", dbTimestamp))
            {
                //assume already converted (i.e., we're in a style sheet and
                //  the shim already converted it; this way we ensure
                //  backwards compatibility)
                return;
            }


            setOutFormat(outFormat);
            setClass(sqlClass);

            try
            {
                if(_outFormat == DSTimeFormat.INT)
                {
                    setDSTimeValueAsInt(dbTimestamp);
                }
                else if(_outFormat == DSTimeFormat.STRING)
                {
                    setDSTimeValueAsString(dbTimestamp);
                }
                else
                {
                    throw new RuntimeException
                        (Text.BROKEN_IF_ELSE);
                }
            }//try
            catch(Exception e)
            {
                _parseError = true;
                _errorMessage = "Invalid database timestamp/date/time value '" +
                                dbTimestamp + "'.";
                addException(e);
            }
        }//else
    }//impl()


    /**
     * Indicates whether precision was lost converting this timestamp
     * to a directory timestamp.
     *
     * @return <code>true</code> if precision was lost;
     * <code>false</code> otherwise
     */
    public
    boolean
    precisionLost()
    {
        return _precisionLost;
    }//precisionLost()


    public
    String
    getDSTime()
    {
        return _time;
    }//getDSTime()


    private
    void
    setDSTimeValueAsString(String trimmed)
        throws ParseException,
               IllegalArgumentException
    {
        //adds support for CCYYMMDDHHMMSSNNNNNNNNN, CCYYMMDD, and HHMMSS
        //  string representations; no precision loss with this type of
        //  conversion; a string representation overcomes integer time
        //  range limitations of eDirectory's native time syntax

        if(_class == java.sql.Timestamp.class)
        {
            try
            {
                Timestamp ts;

                ts = DBLib.NewTimestamp(trimmed);
                setTimestamp(ts);
            }
            catch(IllegalArgumentException iae)
            {
                //defer to custom implementation
                setDSTime(trimmed);
                if(!_customImpl)
                {
                    //throw original exception
                    throw iae;
                }
            }//catch
        }//if
        else if(_class == java.sql.Date.class)
        {
            SimpleDateFormat sdf;

            sdf = new SimpleDateFormat(FORMAT_DATE);
            _time = sdf.format(DBLib.NewDate(trimmed));
        }
        else if(_class == java.sql.Time.class)
        {
            SimpleDateFormat sdf;

            sdf = new SimpleDateFormat(FORMAT_TIME);
            _time = sdf.format(DBLib.NewTime(trimmed));
        }
        else
        {
            throw new RuntimeException
                (Text.BROKEN_IF_ELSE);
        }
    }//setDSTimeValueAsString()


    private
    void
    setDSTimeValueAsInt(String trimmed)
        throws ParseException
    {
        //assume may not convert correctly due to varying formats;
        //  leave to style sheets

        long millis;

        if(_class == java.sql.Timestamp.class)
        {
            //Database precision milliseconds
            //NDS precision seconds

            //Possible data loss in conversion
            try
            {
                Timestamp ts;

                ts = DBLib.NewTimestamp(trimmed);
                millis = ts.getTime();
                if(PrecisionLoss(ts))
                {
                    this._precisionLost = true;
                }

                _time = String.valueOf(Lib.ToSeconds(millis, null));
            }//try
            catch(IllegalArgumentException iae)
            {
                setDSTime(trimmed);
                if(!_customImpl)
                {
                    throw iae;
                }
            }//catch
        }//if
        else if(_class == java.sql.Date.class)
        {
            java.sql.Date date;

            date = DBLib.NewDate(trimmed);
            millis = date.getTime();

            //can't loose precision in conversion
            _time = String.valueOf(Lib.ToSeconds(millis, null));
        }
        else if(_class == java.sql.Time.class)
        {
            Time time;

            time = DBLib.NewTime(trimmed);
            millis = time.getTime();

            //can't loose precision in conversion
            _time = String.valueOf(Lib.ToSeconds(millis, null));
        }
        else
        {
            throw new RuntimeException
                (Text.BROKEN_IF_ELSE);
        }
    }//getDSTimeValueAsInt()

}//class DBTimestamp
