/******************************************************************
Copyright (c) 1996 Novell, Inc. All Rights Reserved.

THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
TREATIES. USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
THAT CONTAINS THIS WORK. PURSUANT TO THE SDK LICENSE AGREEMENT,
NOVELL HEREBY GRANTS TO DEVELOPER A ROYALTY-FREE, NON-EXCLUSIVE
LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS PRODUCT. NOVELL
GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF
DEVELOPER'S PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO
DEVELOPER OR DEVELOPER'S CUSTOMERS WITH RESPECT TO THIS CODE.
******************************************************************/

package com.novell.nds.dirxml.driver.jdbc.db.lock;





/**
 * Database-independent interface used to generate proprietary
 * SQL locking statements based upon a pending standardized
 * SQL DML statement.
 * <p>
 * Implementations should generate the appropriate locking
 * statement(s) that will obtain database locks necessary to
 * prevent blocking of a pending SQL statement.
 * <p>
 * All subclasses MUST have a public constructor with no parameters.
 */
public
interface DBLockStatementGenerator
{

    /**
     * An immutable lock that contains no statements.
     */
    public static final DBLock EMPTY_LOCK = new DBImmutableLock();

    public static final String TYPE_TABLE = "table";
    public static final String TYPE_VIEW  = "view";


    /**
     * Method for caller to supply table name and type info
     * for the pending SQL statement.
     *
     * @param tableName will not be <code>null</code>,
     * empty string, or whitespace
     * @param tableType will be one of the <code>TYPE</code> values
     * defined as part of this interface
     */
    public
    void
    setTable
    (
        String tableName,
        String tableType
    );


    /**
     * Method for caller to supply table primary key info
     * for the table referenced in the pending SQL statement.
     *
     * @param columnName will not be <code>null</code>,
     * empty string, or whitespace
     */
    public
    void
    addPrimaryKeyColumn(String columnName);


    /**
     * Method for caller to supply columns that are listed
     * as part of a pending SELECT, INSERT or UPDATE statement.
     *
     * @param columnName will not be <code>null</code>,
     * empty string, or whitespace
     */
    public
    void
    addListedColumn(String columnName);


    /**
     * Method for caller to supply columns that are listed
     * as part of the WHERE clause for a pending SELECT,
     * UPDATE, or DELETE statement.
     *
     * @param columnName will not be <code>null</code>,
     * empty string, or whitespace
     */
    public
    void
    addWhereColumn(String columnName);


    /**
     * Method for caller to supply the WHERE clause signature
     * of the pending SELECT, UPDATE, or DELETE statement.
     * <p>
     * This should be appened as-is if the lock statement
     * being generated requires a WHERE clause.  The caller
     * will set parameter values.
     * <p>
     *
     * @param whereClause contains a single whitespace character
     * before the word WHERE; will not be <code>null</code>,
     * empty string, or whitespace
     */
    public
    void
    setWhereClause(String whereClause);


    /**
     * Method for caller to request the appropriate database
     * lock for a pending SELECT statement.
     * <p>
     * This method is currently not invoked in any rt.
     * Its use is reserved for future use.
     *
     * @return should not return <code>null</code>
     */
    public
    DBLock
    generateSelectLock();


    /**
     * Method for caller to request the appropriate database
     * lock for a pending INSERT statement.
     *
     * @return should not return <code>null</code>
     */
    public
    DBLock
    generateInsertLock();


    /**
     * Method for caller to request the appropriate database
     * lock for a pending UPDATE statement.
     *
     * @return should not return <code>null</code>
     */
    public
    DBLock
    generateUpdateLock();


    /**
     * Method for caller to request the appropriate database
     * lock for a pending DELETE statement.
     *
     * @return should not return <code>null</code>
     */
    public
    DBLock
    generateDeleteLock();


    /**
     * Method for caller to reset the state of this
     * generator for use with another pending SQL statement.
     */
    public
    void
    reset();

}//interface LockStatementGenerator
